//  2001
//    "  "
//  

//   (   epan.hpp),      
//   ( SOR).     
// ,    .

#include <common.hpp>
#include <epan.hpp>
#include <sor.hpp>
#include <math.h> // for log()
#include <crammer.hpp>
#include <gauss.hpp>

#ifdef USE_REAL
# define VAL(x) x.value()
#else
# define VAL(x) x
#endif

//      
//   T    (rot)
real calc_speed( const matrix & T, uint rot) { 
uint i;
matrix A(T);
real norm;

	for (i=0;i<rot;i++) {
		A=A*T; // or A *= T;
	}
	// now ==^

	// R(A^k) = -ln||A^k||/k
	norm = A.norm1();
	return (real) ((-1)*log( VAL(norm))/rot);
}

//    A,   D,    L   
//  U.
void matrix_break( const matrix & A, matrix & _D, matrix & _L, matrix & _U) 
{
uint n = A.getn();
matrix D(n), L(n), U(n);
uint i, j;

for (i=0;i<n;i++)
	D.set( i, i, A.get(i,i));

for (i=0;i<n;i++)
	for (j=0;j<n;j++) {
		if (i>j) L.set( i, j, A.get(i,j));
		else if (i<j) U.set( i, j, A.get(i,j));
	}

_D = D;
_L = L;
_U = U;

return;

}

#define DIM 20
#define SET_A_VALUES \
	for (i=0;i<DIM;i++) \
		for (j=0;j<DIM;j++) { \
			A.set(i, j, 0); \
			if (i==j) A.set( i, j, 3); \
			if (i==(j-1)) A.set( i, j, -1); \
			if (i==(j+1)) A.set( i, j, -1); \
		}

#define SET_B_VALUES \
	for (i=0;i<DIM;i++) { \
		B.set( i, 1); \
	}


void test_sor() {
	matrix A(DIM);
	vector B(DIM);
	vector *x;
	uint i,j;

	// Put the matrix and vector values
	SET_A_VALUES;
	SET_B_VALUES;
			
	cout << "A: "<<endl;
	cout << A << endl << endl;

	cout << "B: "<<endl;
	cout << B << endl;
	cout << endl;

	// SOR
	cout << endl;

	for (i=1;i<=19;i++) {
		cout << "  SOR[="<<0.1*i<<"]" << endl;

		x = sor_algorithm_fast(A, B, 0.1*i, 0.5*0.00001);
		cout << "X: \n[" <<  *x << "]\n";
		delete x;
		cout << endl;
	}

	cout << "  SOR[="<<1.142<<"]" << endl;

	x = sor_algorithm_fast(A, B, 1.142, 0.5*0.00001);
	cout << "X: \n[" <<  *x << "]\n";
	delete x;
	cout << endl;
	
	// Crammer
	cout << "  Crammer: " << endl;
	x = gauss_algorithm(A, B, 1);
	cout << "X: \n" << *x << endl;
	delete x;
}
