//  2001
//    "  "
//  

//         
//  Ax=B,     Gauss (    ).

#include <common.hpp>
#include <vector.hpp>
#include <matrix.hpp>
#include <real.hpp>
#include <crammer.hpp>

//     (    ).

inline void alg_choice(const uint k, const uint n, matrix & A, vector & B, const uint type) {
	uint j;

//  type==0     
//  type!=0   .

	switch(type) {
		case 0:
			for (j = k; j < n; j++) {
				if (A.get( j, k) != 0) {
					A.swaplines( j, k);
					B.swaplines( j, k);
					break;
				}
			}
			break;
		default: { //  
			real max, tmp;
			uint max_line; //     
			
			max = A.get(k, k);
			max_line = k;
			for (j = k; j < n; j++) {
				//      
				tmp=A.get( j, k);
				if ( tmp > max) {
					max = tmp;
					max_line = j;
				}
			}
			//  
			A.swaplines( max_line, k);
			B.swaplines( max_line, k);
		}
	}		
}

//  type==0     
//  type!=0   .
// Warning: matrix A and vector B are modified.
vector* gauss_algorithm(matrix &A, vector& B, uint type)
{
	int n = A.getn();	// n*n
	int k, j, l;
	real val, s;
	vector* x = new vector(n);
	real mjk;

	/* Eliminate Step */
	for (k = 0; k < n - 1; k++) {
		alg_choice( k, n, A, B, type);

		for (j = k + 1; j < n; j++) {
			// m[j,k] = a[j,k]/a[k,k];
			mjk = val = A.get(j, k) / A.get( k, k);
			A.set( j, k, 0); // This is required!!

			for (l = k + 1; l < n; l++) {
				val =
				    A.get( j, l) - mjk * A.get( k, l);

				A.set( j, l, val);

			}
			val = B.get( j) - mjk * B.get( k);
			B.set( j, val);
		}
	}
	

	/* Back Substitute step */
	x->set(n-1, B.get( n-1) / A.get( n-1, n-1));
	for (k=n-1;k>=0;k--) {
		s = B.get(k);
		for( j = k+1; j<n; j++) {
			// no -= since it must be defined
			s = s - A.get( k, j) * x->get(j);
		}
		x->set( k, s/A.get(k,k));
	}
	return x;
}


#define DIM 50
#define SET_A_VALUES \
	for (i=0;i<DIM;i++) \
		for (j=0;j<DIM;j++) { \
			A.set(i, j, 0); \
			if (i==j) A.set( i, j, 6); \
			if (i==(j-1)) A.set( i, j, 1); \
			if (i==(j+1)) A.set( i, j, 8); \
		}

#define SET_B_VALUES \
	for (i=0;i<DIM;i++) { \
		sum = 0; \
		for (j=0;j<DIM;j++) { \
			sum = sum + A.get( i, j); \
		} \
		B.set( i, sum); \
	}

void test_gauss() {
	
	matrix A(DIM);
	vector B(DIM);
	vector *x;
	uint i,j;
	real sum;

	// Put the matrix values
	SET_A_VALUES;
		
	// put the vector values
	SET_B_VALUES;
			
	cout << "A: "<<endl;
	cout << A << endl << endl;

	cout << "B: "<<endl;
	cout << B << endl;
	cout << endl;
	

	cout << endl;
	x = gauss_algorithm(A, B, 0);
	cout << "  Gauss []: \n[" <<  *x << "]\n";
	delete x;

	SET_A_VALUES;
	SET_B_VALUES;
	
	cout << endl;
	x = gauss_algorithm(A, B, 1);
	cout << "  Gauss [ ]: \n[" <<  *x << "]\n";
	delete x;

	SET_A_VALUES;
	SET_B_VALUES;

	cout << endl;
	cout << "  Crammer: \n";
	x = crammer_algorithm(A, B);
	cout << "[" << *x << "]\n";
	delete x;

}

#define SET_A_VALUES2 \
	A.set( 0, 0, 0.0001); \
	A.set( 0, 1, 1); \
	A.set( 1, 0, 1); \
	A.set( 1, 1, 1)

#define SET_B_VALUES2 \
	B.set( 0, 1); \
	B.set( 1, 2)

#define DIM2 2
void test_gauss2() {
	
	matrix A(DIM2);
	vector B(DIM2);
	vector *x;

	// Put the matrix and vector values
	SET_A_VALUES2;
	SET_B_VALUES2;
			
	cout << "A: "<<endl;
	cout << A << endl << endl;

	cout << "B: "<<endl;
	cout << B << endl;
	cout << endl;

	// Gauss 
	cout << endl;
	x = gauss_algorithm(A, B, 0);
	cout << "  Gauss []: \n[" <<  *x << "]\n";
	delete x;


	// Gauss    
	SET_A_VALUES2;
	SET_B_VALUES2;

	cout << endl;
	x = gauss_algorithm(A, B, 1);
	cout << "  Gauss [ ]: \n[" <<  *x << "]\n";
	delete x;

	// Crammer
	SET_A_VALUES2;
	SET_B_VALUES2;

	cout << endl;
	cout << "  Crammer: \n";
	x = crammer_algorithm(A, B);
	cout << "[" << *x << "]\n";
	delete x;

}


