//  2001
//    "  "
//  

//   (   matrix.hpp),    (matrix)   
//   (,  )  n*n. 
//        .

#include "matrix.hpp"

static matrix NULL_MATRIX(0);

// returns the matrix's size
uint matrix::getn() const
{
	return itsn;
}

#define get_line_index( line) line_index[line]

//         
// ( index)
void matrix::swaplines(const int line1, const int line2)
{
	uint tmp;

	if (line1==line2) return; // no need to do anything
	
	tmp = line_index[ line1];

	line_index[ line1] = line_index[ line2];
	line_index[ line2] = tmp;

}

//   ||A||1
real matrix::norm1() const {
	uint i,j;
	uint n = itsn;
	real max;
	real *sums = new real[n];
	real abs;
	
	for (i=0;i<n;i++) {
		sums[i] = 0;
		for (j=0;j<n;j++) {
			abs = this->get(i,j);
			if (abs<0) abs = abs*(-1);
			sums[i] = sums[i] + abs;
		}
	}

	max = sums[0];
	for (i=0;i<n;i++) {
		if (sums[i] > max) max = sums[i];
	}
	delete sums;
		
	return max;
	
}


void matrix::set( const uint i, const uint j, real val) {
	elem[ (get_line_index(i)*itsn) + j] = val;
}


// returns a matrix element
real matrix::get( const uint i, const uint j) const
{
	return elem[ (get_line_index(i) * itsn) + j];
}

// fills the matrix
void matrix::fill(const real fl)
{
	for (uint i = 0; i < itsn; i++) {
		for (uint j = 0; j < itsn; j++) {
			elem[i * itsn + j] = fl;
		}
	}
}


// fills the diagonial
void matrix::diagfill(const real fl)
{
	for (uint i = 0; i < itsn; i++) {
		for (uint j = 0; j < itsn; j++) {
			if (i == j) {
				elem[get_line_index(i) * itsn + j] = fl;
			} else {
				elem[get_line_index(i) * itsn + j] = 0;
			}
		}
	}
}

// constructor
matrix::matrix(const uint n)
{
	uint i;

	if (n==0) line_index = NULL;
	else line_index = new int[ n];
	for (i = 0; i<n; i++) line_index[i] = i;

	if (n==0) elem=NULL;
	else elem = new real[n * n];
	
	itsn = n;
	for (i = 0; i < n; i++) {
		for (uint j = 0; j < n; j++) {
			elem[ i * itsn + j] = 0;
		}
	}

}

// constructor
matrix::matrix()
{
	line_index = NULL;
	elem = NULL;
	itsn = 0;

}

// destructor
matrix::~matrix()
{
	itsn = 0;
	delete elem;
	delete line_index;
}

// print a matrix
ostream & operator << (ostream & output, const matrix & A) {
	uint n = A.getn();
	for (uint i = 0; i < n; i++) {
		for (uint j = 0; j < n; j++) {
			output << A.get(i, j) << "\t";
		}
		output << endl;
	}
	return output;
}

// constructor
matrix::matrix(const matrix & rhs)
{
	uint i, n;
	
	n = rhs.getn();

	line_index = new int[ n];
	for (i = 0; i<n; i++) line_index[i] = i;

	elem = new real[n * n];
	itsn = n;

	for (i = 0; i < n; i++) {
		for (uint j = 0; j < n; j++) {
			this->set(i, j, rhs.get(i, j));
		}
	}


}

// matrix == matrix
bool matrix::operator == (const matrix & rhs) const {
	
	if (itsn != rhs.getn())
		return 0; // false
	
	for (uint i = 0; i < itsn; i++) {
		for (uint j = 0; j < itsn; j++)
			if (rhs.get(i, j) !=  this->get(i,j)) return 0; // false
	}
	return 1; // true
}

// matrix != matrix
bool matrix::operator != (const matrix & rhs) const {
	
	if ( *this == rhs) return 0; // false
	else return 1; //true
}


// matrix + matrix
matrix matrix::operator + (const matrix & rhs) const {
	matrix B(itsn);
	
	if (itsn != rhs.getn())
		return NULL_MATRIX;
	
	for (uint i = 0; i < itsn; i++) {
		for (uint j = 0; j < itsn; j++)
			B.set(i, j, elem[get_line_index(i) * itsn + j] + rhs.get(i, j));
	}
	return B;
}

// matrix - matrix
matrix matrix::operator - (const matrix & rhs) const {
	matrix B(itsn);
	
	if (itsn != rhs.getn())
		return NULL_MATRIX;
	
	for (uint i = 0; i < itsn; i++) {
		for (uint j = 0; j < itsn; j++)
			B.set(i, j, this->get(i,j) - rhs.get(i, j));
	}
	return B;
}

// matrix = other_matrix
matrix & matrix::operator = (const matrix & rhs) {
	uint i;

	if (this == &rhs)
		return *this;
	delete elem;
	delete line_index;
	
	uint n = rhs.getn();
	itsn = n;
	elem = new real[n * n];

	line_index = new int[ n];
	for (i = 0; i<n; i++) line_index[i] = i;

	for (i = 0; i < n; i++) {
		for (uint j = 0; j < n; j++)
			this->set(i, j, rhs.get(i, j));
	}
	return *this;
}

// matrix * number
matrix matrix::operator * (const real &l) const {
	matrix B(itsn);
	
	for (uint i = 0; i < itsn; i++) {
		for (uint j = 0; j < itsn; j++) {
			B.set(i, j, l * elem[get_line_index(i) * itsn + j]);
		}
	}
	return B;
}


// matrix * matrix
matrix matrix::operator * (const matrix & rhs) const {
	matrix B(itsn);
	
	if (itsn != rhs.getn())
		return NULL_MATRIX;
	
	uint n = itsn;
	real sum = 0;
	for (uint i = 0; i < itsn; i++) {
		for (uint j = 0; j < itsn; j++) {
			sum = 0;
			for (uint x = 0; x < n; x++) {
				sum =
				    sum + elem[get_line_index(i) * itsn + x] * rhs.get(x,
								       j);
			}
			B.set(i, j, sum);
		}
	}
	return B;
}

// matrix * vector
vector matrix::operator * (const vector & rhs) const 
{
	uint n = itsn;
	vector B(n);
	
	if (n != rhs.getn())
		return _NULL_VECTOR;
	
	real sum;
	
	for (uint i = 0; i < n; i++) {
		sum = 0;
		for (uint x = 0; x < n; x++) {
			sum =
			    sum + (this->get( i, x) * rhs.get(x));
		}
		B.set(i, sum);
	}

	return B;
}

//matrix functions

//        a
//   b .
//        delete.
matrix* matrix_step( uint a, uint b, const matrix & A)
{
	uint n = A.getn();
	long bi = -1;
	long bj = -1;
	uint j = 0, i = 0;
	matrix *B = new matrix(n - 1);

	for (i = 0; i < n; i++) {
		if (i != a)
			bi++;
		bj = -1;
		for (j = 0; j < n; j++) {
			if (j != b)
				bj++;
			if (i != a && j != b)
				B->set((uint) bi, (uint) bj,
				      A.get(i, j));
		}
	}
	return B;
}

//  (-1)^(i+j)
inline
static int matrix_pros(uint i, uint j)
{
	i += 1;
	j += 1;

	// % is expensive
	if ((i + j) % 2 == 0) {
		return 1;
	} else {
		return -1;
	}
}

//  (at last!)
//    .   
//        (    ).
real matrix_det(const matrix & A)
{
	uint n = A.getn();
	real b = 0, sum = 0;
	real tmp;
	matrix * mtmp;

	if (A.getn() == 2) {
		return ((A.get(0, 0) * A.get(1, 1)) - (A.get(1, 0) * A.get(0, 1)));
	} else {
		if (A.getn() == 0 || A.getn() == 1) {
			return 0;
		} else {
			for (uint j = 0; j < n; j++) {
				tmp = A.get(0, j);

				//     ,   
				//       .
				if (tmp!=0) {
					mtmp = matrix_step(0, j, A);
					b = matrix_det( *mtmp) * matrix_pros(0, j) * tmp;
					delete mtmp;
				} else b = 0;
				
				sum = sum + b;
			}
			return sum;
		}
	}
}

//anastrofos pinakas
matrix matrix_invert(const matrix & A)
{
	uint n = A.getn();
	matrix B(n);	
	
	for (uint i = 0; i < n; i++) {
		for (uint j = 0; j < n; j++) {
			B.set(j, i, A.get(i, j));
		}
	}
	return B;
}

//antistrofos pinakas!
matrix matrix_reverse(const matrix & A)
{
	real a = 0, det = matrix_det(A);
	uint n = A.getn();
	matrix B(n);
	matrix *mtmp;
	
	if (det == 0 || n<=1)
		return NULL_MATRIX;
	
	if (n == 2) {
		B.set(0, 0, A.get(1, 1) / det);
		B.set(0, 1, (A.get(0, 1) * (-1)) / det);
		B.set(1, 0, (A.get(1, 0)* (-1)) / det);
		B.set(1, 1, A.get(0, 0) / det);
	} else {
		for (uint i = 0; i < n; i++) {
			for (uint j = 0; j < n; j++) {
				mtmp = matrix_step(i, j, A);
				a = ( (matrix_det( *mtmp)) * matrix_pros(i, j)) / det;
				delete mtmp;
				B.set(j, i, a);
			}
		}
	}

	return B;

}
