//  2001
//    "  "
//  

//   (   sor.hpp),     SOR
//  . H    sor_algorithm(),  
//    SOR    .

#include <sor.hpp>

//    L  SOR . 
// 0  .
int calc_Lw( matrix & _Lw, const real& w, const matrix& D, const matrix& L, const matrix &U) {

	//  SOR   w < 2 && w > 0
	if (w < 0 || w > 2) return -1; // error

	// Lw = (D-wL)^(-1) * [(1-w)*D+wU]
	_Lw = matrix_reverse(D-(L*w)) * ((D*((real)1-w)) + (U*w));
	
	return 0; // finished

}

//    c  SOR . 
// 0  .
int calc_cw( vector & _cw, const real& w, const matrix &D, const matrix &L, const vector & b) {

	//  SOR   w < 2 && w > 0
	if (w < 0 || w > 2) return -1; // error

	// cw = w(D-wL)^(-1) * b
	_cw = (matrix_reverse(D-(L*w))*w) * b;
	
	return 0; // finished

}

//  0         
//    (a,b).  prec(ision)    0.5*10^(-m),
//  m      .
int check_precision( const vector & x_prev, const vector &x, real prec) {
	vector dif;

	//   
	dif = x - x_prev;
	if ( dif.norm()/x.norm() < prec)
		return 0;
	else
		return 1;
		 
}

//   SOR      .
vector *sor_algorithm_fast( const matrix & A, const vector & B, const real& w, const real &prec) {
int i,j;
int n = B.getn();
vector x(n), old_x;
uint loops = 0;
vector * ret = new vector(x);
real sum1, sum2, val;

	x.fill(1);
	do {
		old_x = x;
		for (i=0;i<n;i++) {
			
			sum1 = 0;
			for(j=0;j<i;j++) {
				sum1 = sum1 + A.get(i,j)*x.get(j);
			}

			sum2 = 0;
			for(j=i+1;j<n;j++) {
				sum2 = sum2 + A.get(i,j)*old_x.get(j);
			}
			
			val = (1-w)*old_x.get(i) + (w*(B.get(i) - sum1 - sum2)) / A.get(i,i);
			x.set( i, val);
		}
		loops++;
	} while( check_precision( x, old_x, prec)!=0);

	cout << "Loops in SOR algorithm: " << loops <<endl;
	*ret = x;
	return ret;
}
