//  2001
//    "  "
//  

// :
//    vector      
// .      (+,-),
//    .

#define NULL_OK
#include "vector.hpp"

vector _NULL_VECTOR(0);

// returns the vector's size
uint vector::getn() const
{
	return itsn;
}

void vector::swaplines(const int line1, const int line2)
{
	real tmp;

	if (line1==line2) return; // no need to do anything
	
	tmp = elem[line1];

	elem[ line1] = elem[ line2];
	elem[ line2] = tmp;

}

void vector::set(const uint i, real val) {
 elem[ i]=val;
}


// returns a vector element
real vector::get(const uint i) const
{
	return elem[ i];
}

real vector::norm1() const
{
	real max, abs;
	uint i;
	
	max = elem[0];
	if (max<0) max = max*(-1);
	
	for (i=0;i<itsn;i++) {
		abs = elem[i];
		if (abs<0) abs = abs * (-1);
		
		if (elem[i] > max) max = elem[i];
	}
	
	return max;
	
}

real vector::norm() const
{
	real max, abs;
	uint i;
	max = 0;
		
	for (i=0;i<itsn;i++) {
		abs = elem[i]; // abs == |elem[i]|
		if (abs<0) abs = abs * (-1);
		
		if (elem[i] > max) max = max + elem[i];
	}
	
	return max;
	
}

// fills the matrix
void vector::fill(const real fl)
{
	for (uint j = 0; j < itsn; j++) {
		elem[ j] = fl;
	}
}


// constructor
vector::vector(const uint n)
{
	uint i;

	if (n==0) elem = NULL;
	else elem = new real[n];
	
	for (i = 0; i < n; i++) {
		elem[ i] = 0;
	}
	itsn = n;

}

// constructor
vector::vector()
{
	// this creates an unusable vector
	// you must initialize it later
	elem = NULL;
	itsn = 0;

}

// destructor
vector::~vector()
{
	itsn = 0;
	delete elem;
}

// print a vector
ostream & operator << (ostream & output, const vector & A) {
	uint n = A.getn();
	for (uint i = 0; i < n; i++) {
		output << A.get(i) << "\t";
	}
	return output;
}

// constructor
vector::vector(const vector & rhs)
{
	uint i, n;
	
	itsn = n = rhs.getn();

	elem = new real[n];

	for (i = 0; i < n; i++) {
		elem[ i] = rhs.get(i);
	}
}


// vector + vector
vector vector::operator + (const vector & rhs) const {
	if (itsn != rhs.getn())
		return vector(0);

	vector B(itsn);

	for (uint i = 0; i < itsn; i++) {
		B.set(i, elem[i] + rhs.get(i));
	}
	return B;
}

// vector - vector
vector vector::operator - (const vector & rhs) const {
	if (itsn != rhs.getn())
		return vector(0);

	vector B(itsn);

	for (uint i = 0; i < itsn; i++) {
		B.set( i, (elem[i] - rhs.get(i)) );
	}
	return B;
}

// vector = other_vector
vector & vector::operator = (const vector & rhs) {
	if (this == &rhs)
		return *this;

	delete elem;

	uint n = rhs.getn();
	itsn = n;
	elem = new real[n];

	for (uint i = 0; i < n; i++) {
		elem[i] = rhs.get(i);
	}
	return *this;
}

// vector * number
vector vector::operator * (const real l) const {
	vector B(itsn);
	for (uint i = 0; i < itsn; i++) {
		B.set(i, l * elem[i]);
	}
	return B;
}


// vector * vector
//  
real vector::operator * (const vector& l) const {
real ret = 0;

	for (uint i = 0; i < itsn; i++) {
		ret = ret + (this->get(i)*l.get(i));
	}
	return ret;
}


